# Copyright 2016, 2017 Cumulus Networks, Inc.  All Rights reserved.
#
# This software is subject to the Cumulus Networks End User License Agreement
# available at the following locations:
#
# Internet: https://cumulusnetworks.com/downloads/eula/latest/view/
# Cumulus Linux systems: /usr/share/cumulus/EULA.txt

from __future__ import print_function

import os
import re
import shutil
import subprocess

try:
    from yaml import CLoader as Loader, CDumper as Dumper
except ImportError:
    from yaml import Loader, Dumper

try:
    import StringIO
except ImportError:
    from io import StringIO

import ipaddr
import redis
import yaml

from netq_apps.modules import RC_FAIL, RC_SUCCESS


def rc_success(msg=None):
    return RC_SUCCESS, msg if msg else ''


def rc_fail(msg=None):
    return RC_FAIL, msg if msg else 'Failure.'


def write_file(inp, path, **kwargs):
    try:
        expath = os.path.expandvars(path)
        exdir  = os.path.dirname(expath)

        if not os.path.exists(exdir):
            try:
                os.makedirs(exdir)
            except OSError:
                pass

        if os.path.exists(expath):
            if os.path.isfile(expath):
                try:
                    os.remove(expath)
                except:
                    pass

            elif os.path.isdir(expath):
                shutil.rmtree(expath,ignore_errors=True)

        with open(expath, mode='w') as cfg:
            if isinstance(inp, StringIO.StringIO):
                return rc_success(cfg.write(inp.getvalue()))
            else:
                return rc_success(cfg.write(inp))
    except IOError as ioe:
        return rc_fail(ioe.message)


def read_file(path, **kwargs):
    expath = os.path.expandvars(path)
    if os.path.exists(expath) and os.path.isfile(expath):
        with open(os.path.expandvars(path), 'r') as cfg:
            try:
                return rc_success(cfg.read())
            except IOError as ioe:
                return rc_fail(ioe)
    return rc_fail('%s does not exist' % (expath))

def run(cmd):
    with open(os.devnull, 'w') as fnull:
        try:
            return rc_success(subprocess.check_output(cmd.split(), stderr=fnull))
        except subprocess.CalledProcessError:
            return rc_fail('Failed to run %s ' % (cmd))


# From netq_apps/modules/core/netq/daemon.pyx
def systemd_service(cmd, service):
    '''Common systemctl commands function for daemons'''
    legit_cmds = ['start', 'stop', 'status', 'restart', 'reload', 'disable', 'enable', 'is-active']
    systemctl = '/bin/systemctl'

    if cmd not in legit_cmds:
        return rc_fail('Command not recognized: %s' % (cmd))

    # if cmd != 'status' and os.getuid() != 0:
    #    return rc_fail('Please run this command as root.')

    with open(os.devnull, 'w') as fnull:

        if cmd in ['start', 'restart']:
            cmds = ['enable', 'reset-failed', cmd]
        elif cmd == 'stop':
            cmds = [cmd, 'disable']
        else:
            cmds = [cmd]

        ret = ['Service: %s' % service]
        for cmd in cmds:
            try:
                ret.append('Action: %s' % cmd.title())
                ret.append(rc_success(subprocess.check_output([systemctl, cmd, service], stderr=fnull)))
            except subprocess.CalledProcessError as cpe:
                ret.append(rc_fail(cpe.output))

        return ret


def redis_wrap((host, port), fcn, *args, **kwargs):
    try:
        conn = redis.StrictRedis(host, port=port, socket_timeout=2)
        return fcn(conn, *args, **kwargs)
    except redis.ConnectionError as ce:
        return rc_fail('REDIS connection error %s %s' % (host, ce))
    except redis.exceptions.RedisError as re:
        return rc_fail('REDIS error %s' % (re))
    except OSError as ose:
        return rc_fail('REDIS OS error %s' % (ose))


__VALID_HOSTNAME_RE='(?!-)[A-Z\d-]{1,63}(?<!-)$'


def is_valid_hostname(hostname):
    if len(hostname) > 255: return False
    if hostname[-1] == ".": hostname = hostname[:-1]
    valid_seg = re.compile(__VALID_HOSTNAME_RE, re.IGNORECASE)
    return all(valid_seg.match(x) for x in hostname.split("."))


def get_wildcard_ips(ips):
    if not ips:
        return rc_fail('No IPs provided.')
    servers = set()  # verify replicas
    for ip in str(ips).split():
        try:
            ipaddr.IPNetwork(unicode(ip))
            servers.add(ip)
        except ValueError as ex:
            if is_valid_hostname(ip):
                servers.add(ip)
            else:
                return rc_fail(ex.message)

    return rc_success(list(servers)) if len(servers) else rc_fail('No IPs provided.')


def save_config(loc, settings):
    try:
        sfile = open(loc,'w')
    except IOError:
        pass
    else:
        with sfile:
            try:
                yaml.safe_dump(settings,sfile)
            except yaml.YAMLError:
                pass

def get_config(loc):
    try:
        sfile = open(loc,'r')
    except IOError:
        return {}
    else:
        with sfile:
            try:
                return yaml.safe_load(sfile)
            except yaml.YAMLError as error:
                return { 'msg':'Error parsing: %s' % loc, 'error':'%s' % str(error) }
